@@ -226,17 +226,17 @@ def eval_adverb_over(f, a):
     # https://docs.cupy.dev/en/stable/reference/ufunc.html
     # TODO: can we use NumPy reduce when CuPy backend primary?
     if isinstance(wrapped, KGOp):
-        if safe_eq(wrapped.a,'+'):
-            return np.sum(a)
+        if safe_eq(wrapped.a,'+') and hasattr(np.add,'reduce'):
+            return np.add.reduce(a) if len(a.shape) > 1 else np.sum(a)
         elif safe_eq(wrapped.a, '-') and hasattr(np.subtract,'reduce'):
             return np.subtract.reduce(a)
         elif safe_eq(wrapped.a, '*') and hasattr(np.multiply,'reduce'):
             return np.multiply.reduce(a)
         elif safe_eq(wrapped.a, '%') and hasattr(np.divide,'reduce'):
             return np.divide.reduce(a)
-        elif safe_eq(wrapped.a, '&'):
+        elif safe_eq(wrapped.a, '&') and len(a.shape) == 1:
             return np.min(a)
-        elif safe_eq(wrapped.a, '|'):
+        elif safe_eq(wrapped.a, '|') and len(a.shape) == 1:
             return np.max(a)
         elif safe_eq(wrapped.a, ','):
             if isinstance(a,str):
